#!/usr/bin/env python
# Description = Maps each domains to their corresponding domains from InterPro

import argparse
import os

__description__ = ("Maps each domains to their corresponding domains from "
                   "InterPro")


def main():
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument("go_path")
    parser.add_argument("interpro_to_go")
    parser.add_argument("interpro_data")
    parser.add_argument("cdd_data")
    args = parser.parse_args()
    
    map_domain_to_go = MapDomainToGo(
        args.go_path, args.interpro_to_go,
        args.interpro_data, args.cdd_data)
    map_domain_to_go.read_obo_file()
    map_domain_to_go.map_interpro_to_domains()
    map_domain_to_go.read_mapped_go_data()
    map_domain_to_go.map_cdd_to_go()


class MapDomainToGo(object):

    def __init__(self, go_path,
                 interpro_to_go,
                 interpro_data,
                 cdd_data):
        self._go_path = go_path
        self._interpro_to_go = interpro_to_go
        self._interpro_data = interpro_data
        self._cdd_data = cdd_data
        
        self._go_info_dict = {}
        self._interpro_domain_dict = {}
        self._domain_interpro_dict = {}
        self._interpro_go_dict = {}
        
    def read_obo_file(self):
        '''parses and reads the GO obo file'''
        out_fh = open(self._go_path+'/go_obo_table.csv', 'w')
        for go_file in os.listdir(self._go_path):
            go_fh = open(self._go_path+'/'+go_file, 'r')
            if '.obo' in go_file:
                for entry in go_fh.read().split('[Term]'):
                    if 'id: GO:' in entry:
                        info_dict = {}
                        for info in entry.split('\n'):
                            if info.startswith('id: GO:'):
                                info_dict["go_id"] = info.split(
                                    ': ')[1].strip()
                            if "name:" in info:
                                info_dict["name"] = info.split(
                                    ': ')[1].strip()
                            if "namespace:" in info:
                                if "biological_process" in info:
                                    info_dict["namesp"] = 'bp'
                                elif "cellular_component" in info:
                                    info_dict["namesp"] = 'cc'
                                elif "molecular_function" in info:
                                    info_dict["namesp"] = 'mf'
                        out_fh.write("%s:%s\t%s\n" % (
                                info_dict["namesp"], info_dict["go_id"],
                                info_dict["name"]))
                        self._go_info_dict[info_dict["go_id"]] = (
                            "%s:%s [%s]" % (info_dict["namesp"],
                            info_dict["go_id"], info_dict["name"]))
            go_fh.close()
        out_fh.close()
        return self._go_info_dict
    
    def map_interpro_to_domains(self):
        '''maps ipr ids to domain ids'''
        with open(self._interpro_data, 'r') as in_fh:
            for entry in in_fh:
                ipr_id = entry.split('\t')[0]
                domain_ids = entry.strip().split('\t')[-1]
                self._interpro_domain_dict[ipr_id] = domain_ids
                if '|' in domain_ids:
                    for domains in domain_ids.split('|'):
                        self._domain_interpro_dict[domains] = ipr_id
                else:
                    self._domain_interpro_dict[domain_ids] = ipr_id
        return self._interpro_domain_dict, self._domain_interpro_dict

    def read_mapped_go_data(self):
        '''maps domains to go id'''
        with open(self._interpro_to_go, 'r') as interpro_to_go_fh:
            for mapped_entry in interpro_to_go_fh:
                if "InterPro:" in mapped_entry:
                    ipr_id = mapped_entry.split('InterPro:'
                                                )[1].split(' ')[0]
                    go_id = mapped_entry.split(';')[-1].split(
                        'GO:')[1].strip()
                    self._interpro_go_dict.setdefault(
                        ipr_id, []).append("GO:%s" % go_id)
        with open(self._go_path+'/mapped_interpro_to_go.csv',
                  'w') as map_out_fh:
            with open(self._go_path+'/unmapped_interpro_to_go.csv',
                      'w') as unmap_out_fh:
                for ipr in set(self._interpro_domain_dict.keys()):
                    if ipr in set(self._interpro_go_dict.keys()):
                        if len(self._interpro_go_dict[ipr]) > 1:
                            go_list = []
                            for go in self._interpro_go_dict[ipr]:
                                try:
                                    go_list.append(self._go_info_dict[go])
                                except KeyError:
                                    go_list.append('%s: No description' % go)
                            map_out_fh.write("%s\t%s\t%s\n" % (
                                ipr, self._interpro_domain_dict[ipr],
                                ','.join(go_list)))
                        else:
                            try:
                                map_out_fh.write("%s\t%s\t%s\n" % (
                                ipr, self._interpro_domain_dict[ipr],
                                self._go_info_dict[
                                    self._interpro_go_dict[ipr][0]]))
                            except:
                                map_out_fh.write("%s\t%s\t%s\n" % (
                                ipr, self._interpro_domain_dict[ipr],
                                'No description'))
                    else:
                        unmap_out_fh.write("%s\t%s\n" % (
                            ipr, self._interpro_domain_dict[ipr]))
        return self._interpro_go_dict
    
    def map_cdd_to_go(self):
        '''maps cdd to go and interpro'''
        map_go_fh = open(
            self._go_path+'/mapped_cdd_to_go.csv', 'w')
        unmap_go_fh = open(
            self._go_path+'/unmapped_cdd_to_go.csv', 'w')
        map_ipr_fh = open(
            self._go_path+'/mapped_cdd_to_interpro.csv', 'w')
        unmap_ipr_fh = open(
            self._go_path+'/unmapped_cdd_to_interpro.csv', 'w')
        interpro_path = '/'.join(self._interpro_to_go.split('/')[0:-1])
        ipr_map_cdd_length = open(
            interpro_path+'/mapped_interpro_to_cdd_length.csv', 'w')
        with open(self._cdd_data, 'r') as cdd_data_fh:
            for cdd_entry in cdd_data_fh:
                cdd_id = cdd_entry.split('\t')[0]
                cdd_db_id = cdd_entry.split('\t')[1]
                cdd_length = cdd_entry.strip().split('\t')[-1]
                if 'pfam' in cdd_db_id:
                    cdd_db_id = cdd_db_id.replace('pfam', 'PF')
                if 'smart' in cdd_db_id:
                    cdd_db_id = cdd_db_id.replace('smart', 'SM')
                if cdd_db_id in set(self._domain_interpro_dict.keys()):
                    ipr = self._domain_interpro_dict[cdd_db_id]
                    ipr_map_cdd_length.write("%s\t%s\t%s\t%s\t%s\n" % (
                        ipr, self._interpro_domain_dict[ipr],
                        cdd_id, cdd_db_id, cdd_length))
                    map_ipr_fh.write("%s\t%s\t%s\t%s\n" % (
                        cdd_id, cdd_db_id,
                        ipr, self._interpro_domain_dict[ipr]))
                    try:
                        if self._interpro_go_dict[ipr]:
                            if len(self._interpro_go_dict[ipr]) > 1:
                                go_list = []
                                for go in self._interpro_go_dict[ipr]:
                                    go_list.append(self._go_info_dict[go])
                                map_go_fh.write("%s\t%s\t%s\n" % (
                                    cdd_id, cdd_db_id, ','.join(go_list)))
                            else:
                                map_go_fh.write("%s\t%s\t%s\n" % (
                                    cdd_id, cdd_db_id, self._go_info_dict[
                                        self._interpro_go_dict[ipr][0]]))
                    except KeyError:
                        unmap_go_fh.write("%s\t%s\n" % (
                            cdd_id, cdd_db_id))
                else:
                    unmap_ipr_fh.write("%s\t%s\n" % (cdd_id, cdd_db_id))
        ipr_map_cdd_length.close()
        map_go_fh.close()
        unmap_go_fh.close()
        map_ipr_fh.close()
        unmap_ipr_fh.close()

if __name__ == '__main__':
    
    main()



#!/usr/bin/env python
# Description = RNA related InterPro domains

import re


class RnaRelatedIPRSelection(object):
    def __init__(self, keywords_file,
                 ipr_whole_data_file,
                 interpro_mapped_cdd,
                 domain_data_path):
        self._keywords_file = keywords_file
        self._ipr_whole_data_file = ipr_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._domain_data_path = domain_data_path
        
        self.ipr_whole_data_list = []
        self._ipr_dict = {}
        self._mapped_interpro_length = {}
    
    def select_ipr_domains(self):
        '''To call from apricot'''
        self.read_keyword_file()
        self.read_interpro_mapped_cdd_file()
        self.read_ipr_whole_data_file()
        self.create_rna_related_domain_file()
        
    def read_keyword_file(self):
        '''reads keywords for domain selection'''
        self.keyword_list = [rna_keyword.strip()
                             for rna_keyword in open(
                                self._keywords_file)]
        return self.keyword_list
    
    def read_interpro_mapped_cdd_file(self):
        '''Parses interpro cdd mapped file to extract common information'''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                ipr_id = entry.split('\t')[0]
                ipr_members = entry.split('\t')[1]
                cdd_id = entry.split('\t')[2]
                cdd_member = entry.split('\t')[3]
                domain_length = entry.split('\t')[4].strip()
                self._mapped_interpro_length[cdd_member] = domain_length
        self._mapped_interpro_length
        
    def read_ipr_whole_data_file(self):
        '''Parses InterPro annotation data from table'''
        with open(self._ipr_whole_data_file, 'r') as in_fh:
            for ipr_id_entry in in_fh:
                self.ipr_whole_data_list.append(ipr_id_entry.strip())
                self._ipr_dict[ipr_id_entry.strip()] = ipr_id_entry.strip(
                    ).split('\t')[2]
        return self.ipr_whole_data_list, self._ipr_dict
        
    def create_rna_related_domain_file(self):
        '''Creates RNA related domain list'''
        self._keyword_annotation_dict = {}
        self._rna_related_domain = []
        for ipr_entry in self.ipr_whole_data_list:
            for keyword in self.keyword_list:
                if ' ' in keyword:
                    key_list = []
                    for each_key in keyword.split(' '):
                        key_list.append(each_key)
                    match = re.search(r'\b%s*|\W%s\b'%(key_list[0].lower(),
                                                       key_list[1].lower()),
                                      self._ipr_dict[ipr_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(ipr_entry)
                else:
                    match = re.search(r'\b%s\b'%keyword, self._ipr_dict[ipr_entry])
                    if match:
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(ipr_entry)
        for fkeyword in self.keyword_list:
            fkeyword = fkeyword.replace(' ', '_')
            with open(self._domain_data_path + '/' +
                      fkeyword + '_related_ipr_ids.tab',
                      'w') as key_fh:
                if self._keyword_annotation_dict.get(fkeyword):
                    for each_entry in self._keyword_annotation_dict[fkeyword]:
                        each_entry = each_entry.replace(each_entry.split(
                            '\t')[3], " ".join(each_entry.split('\t')[3].split(
                            ))).replace(';', ',')
                        cdd_id = each_entry.split('\t')[1]
                        if cdd_id in set(self._mapped_interpro_length.keys()):
                            length = self._mapped_interpro_length[cdd_id]
                        else:
                            length = 'NA'
                        self._rna_related_domain.append("%s\t%s" % (
                            each_entry, length))
                        key_fh.write("%s\t%s\n" % (each_entry, length))
        uniq_rna_related_domains = list(set(self._rna_related_domain))
        with open(self._domain_data_path+'/all_rna_related_ipr_data.tab',
                  'w') as rna_related_domain_file:
            for domain_entry in uniq_rna_related_domains:
                rna_related_domain_file.write('%s\n' % str(domain_entry))



#!/usr/bin/env python
# Description = Merges domains files selected from CDD and InterPro

import os
import sys


def merge_domain_data(cdd_domains, ipr_domains, merged_file):
    with open(merged_file, 'w') as out_fh:
        header = '\t'.join(['ReferenceId', 'DomainId',
                            'ShortName', 'FullName',
                            'Members', 'Length', 'SelectionTerm'])+'\n'
        out_fh.write(header)
        if os.path.exists(cdd_domains):
            with open(cdd_domains, 'r') as cdd_fh:
                out_fh.write(cdd_fh.read()+'\n')
        if os.path.exists(ipr_domains):
            with open(ipr_domains, 'r') as ipr_fh:
                out_fh.write(ipr_fh.read()+'\n')

if __name__ == '__main__':
    cdd_domains = sys.argv[1]
    ipr_domains = sys.argv[2]
    merged_file = sys.argv[3]
    merge_domain_data(cdd_domains, ipr_domains, merged_file)



#!/usr/bin/env python
# Description = Compiles InterProScan data from XML files to table

import sys

try:
    import xml.etree.ElementTree as ET
except ImportError:
    print('Python package xmlis missing. Please install/update.')
    sys.exit(0)
import re

interpro_analysis_path = sys.argv[1]
interpro_xml = sys.argv[2]

interproid = open(
    interpro_analysis_path+'/interproid.tbl', 'w')
only_id_ipr_xml = open(
    interpro_analysis_path+'/only_id_ipr_xml.txt', 'w')
ipr_data_removed = open(
    interpro_analysis_path+'/ipr_data_removed.txt', 'w')

tree = ET.parse(interpro_xml)
root = tree.getroot()

for data in root.findall('interpro'):
    abstract_list = []
    member_db_list = []
    
    ipr_id = data.attrib.get('id')
    name = data.find('name').text
    only_id_ipr_xml.write('%s\t%s\n' % (ipr_id, name))
    
    member_list = data.findall('./member_list/*')
    external_doc = data.findall('./external_doc_list/*')
    db_list = [member_list, external_doc]
    for db_data in db_list:
        for db_elem in db_data:
            member_db_list.append(
                db_elem.attrib.get('dbkey'))
    whole_external_members = "|".join(
        sorted(list(set(member_db_list))))
    main_abstract = data.find('abstract').text
    abstract_list.append(main_abstract.replace(
        '\n', ' ').replace('[', ''))
    abstract_data = data.findall('./abstract/*')
    for abs_elem in abstract_data:
        abs_unordered = ET.tostring(abs_elem).strip()
        abs_ordered = str(abs_unordered).replace(
            "b'<p>", "").replace("</p>'", "")
        for each_abs_elem in abs_ordered.split('\n'):
            try:
                abs_list_elt = re.sub(
                '<[A-Za-z\/][^>]*>', '', each_abs_elem).replace(
                "\\n", "").replace("'b'.", "").replace(
                    "]", "").replace("[", "").replace(
                    "b')", "").replace("b'", "").replace(
                "'", "").replace(" ,", " ").replace(
                "()", "").replace(" .", ".").replace(",,", "")
                if len(abs_list_elt) > 2:
                    abstract_list.append(abs_list_elt)
            except AttributeError:
                pass
    full_abstract = (''.join(abstract_list))
    whole_abstract = re.sub(r'\s+', ' ', full_abstract)
    for external_members in member_db_list:
        try:
            interproid.write('\t'.join([ipr_id, external_members,
                          name, whole_abstract,
                          whole_external_members])+'\n')
        except UnicodeEncodeError:
            interproid.write('\t'.join([ipr_id, external_members,
                          name, repr(whole_abstract),
                          whole_external_members])+'\n')
        
    
for deleted_data in root.findall('deleted_entries'):
    for ipr_id_info in deleted_data:
        ipr_id = ipr_id_info.attrib.get('id').strip()
        ipr_data_removed.write('%s\n'%ipr_id)
ipr_data_removed.close()
only_id_ipr_xml.close()
interproid.close()



#!/usr/bin/env python
# Description = Predict 3 or 8 state secondary structure using RaptorX

import os
import subprocess


class RaptorxSecstrAnalysis(object):
    def __init__(self, selected_proteins, raptorx_path,
                 fasta_path, outpath):
        self._selected_proteins = selected_proteins
        self._raptorx_path = raptorx_path
        self._fasta_path = fasta_path
        self._outpath = outpath
        
        self._selected_protein_set = set()
        
    def streamline_raptorx_secstr_analysis(self):
        '''To call from apricot'''
        self.parse_selected_data()
        self.run_raptorx_analysis()
        self.create_job_completion_file()
        
    def parse_selected_data(self):
        '''Parses selected data for uid'''
        with open(self._selected_proteins, 'r') as in_fh:
            for entry in in_fh:
                if not entry.startswith('Entry'):
                    self._selected_protein_set.add(entry.split('\t')[0])
        return self._selected_protein_set
    
    def run_raptorx_analysis(self):
        '''Runs RaptorX on the selected uids for 8-state secondary structure
        prediction

        '''
        for files in os.listdir(self._fasta_path):
            if files.split('.')[0] in self._selected_protein_set:
                print("RaptorX 8-state secondary structure analysis for %s" %
                      files)
                subprocess.Popen(
                    ["perl %s %s/%s" %
                     (self._raptorx_path, self._fasta_path, files)],
                    shell=True).wait()
                subprocess.Popen(["mv *.ss* %s" % self._outpath],
                                 shell=True).wait()
                subprocess.Popen(["mv *.horiz %s" % self._outpath],
                                 shell=True).wait()
                subprocess.Popen(["rm -rf tmp*.%s" % files.split('.')[0]],
                                 shell=True).wait()
                
    def create_job_completion_file(self):
        with open(self._outpath+'/raptorx_analysis.txt', 'w') as out_fh:
            out_fh.write("Secondary structures for the selected proteins are "
                         "generated by RaptorX.\n")
            out_fh.write("The files generated by the analysis:.\n")
            out_fh.write('\n'.join(os.listdir(self._outpath)))



#!/usr/bin/env python
# Description = Download UniProt based proteome data for a taxonomy id

import sys
try:
    from urllib.request import urlopen
except ImportError:
    print('Python package urllib is missing. Please install/update.\n')
    sys.exit(0)


def format_uniprot_table(proteome_table, uniprot_link):
    '''Downloads protein information
    table from UniProt database for
    the selected taxonomy id'''
    try:
        response = urlopen(uniprot_link)
        for entry in str(response.read()).split('\\n'):
            if not entry == "'" and not entry == '"':
                if not entry.startswith(
                        "b'Entry") and not entry.startswith('b"Entry'):
                    proteome_table.write("%s\n" % '\t'.join(
                        list(entry.split('\\t'))))
        print('"\nDownloaded protein information using UniProt link: %s\n"' % (
            uniprot_link))
    except:
        print(
            "UniProt entry is apparently deleted, please check: %s"
            % uniprot_link)



#!/usr/bin/env python
# Description = Select domain reference set based on user provided keywords

import re


class KeywordBasedDomainSelection(object):
    def __init__(self, keywords_file,
                 pfam_domain_file,
                 cdd_whole_data_file,
                 interpro_whole_data_file,
                 interpro_mapped_cdd,
                 cdd_domain_path,
                 ipr_domain_path):
        self._keywords_file = keywords_file
        self._pfam_domain_file = pfam_domain_file
        self._cdd_whole_data_file = cdd_whole_data_file
        self._interpro_whole_data_file = interpro_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._cdd_domain_path = cdd_domain_path
        self._ipr_domain_path = ipr_domain_path
        
        self._keyword_list = []
        self._pfam_domain_dict = {}
        self._mapped_cdd_members = {}
        self._mapped_ipr_legnth = {}
        self._selected_cdd_entry_dict = {}
        self._selected_ipr_entry_dict = {}
    
    def select_cdd_and_ipr_domains(self):
        self.read_keyword_file()
        self.read_pfam_domain_file()
        self.read_interpro_mapped_cdd_file()
        self.map_cdd_data_to_keywords()
        self.map_ipr_data_to_keywords()
        
    def select_cdd_domains(self):
        self.read_keyword_file()
        self.read_pfam_domain_file()
        self.read_interpro_mapped_cdd_file()
        self.map_cdd_data_to_keywords()
        
    def select_ipr_domains(self):
        self.read_keyword_file()
        self.read_pfam_domain_file()
        self.read_interpro_mapped_cdd_file()
        self.map_ipr_data_to_keywords()
    
    def _string_search(self, keyword, annotation):
        if ' ' not in keyword:
            if '. ' not in annotation:
                if ',' in str(annotation):
                    annotation = ' '.join(str(annotation).split(','))
                keyword_match = self._match_keys_in_annotation(
                    keyword, annotation)
                if keyword_match:
                    return True
            else:
                for each_annotation in annotation.split('. '):
                    keyword_match = self._match_keys_in_annotation(
                        keyword, each_annotation)
                    if keyword_match:
                        return True
        else:
            ref_kw_list = keyword.lower().split(' ')
            if '. ' not in annotation:
                if ',' in str(annotation):
                    annotation = ' '.join(str(annotation).split(','))
                new_kw_list = []
                for each_word in ref_kw_list:
                    current_search = self._match_keys_in_annotation(
                        each_word, annotation)
                    if current_search:
                        new_kw_list.append(each_word)
                keyword_match = ref_kw_list == new_kw_list
                if keyword_match:
                    return True
            else:
                for each_annotation in annotation.split('. '):
                    
                    new_kw_list = []
                    for each_word in ref_kw_list:
                        current_search = self._match_keys_in_annotation(
                            each_word, each_annotation)
                        if current_search:
                            new_kw_list.append(each_word)
                    keyword_match = ref_kw_list == new_kw_list
                    if keyword_match:
                        return True
        return False
    
    def _match_keys_in_annotation(self, keyw, annotation):
        if '#' not in keyw:
            annotation = annotation.lower().replace(
                    '_', ' ').replace('-', ' ')
            if ' ' in annotation:
                for each_annotation in list(annotation.split(' ')):
                    if re.findall(r"(?:\s|^)%s(?=\s|$)" %
                        keyw.lower(), each_annotation):
                        return True
            else:
                if re.findall(r"(?:\s|^)%s(?=\s|$)" %
                    keyw.lower(), annotation):
                    return True
        else:
            keyw = keyw.replace('#', str('\w*'))
            annotation = annotation.lower().replace(
                '_', ' ').replace('-', ' ')
            for each_annotation in list(annotation.split(' ')):
                if re.findall(r'%s' % keyw.lower(), each_annotation):
                    return True
            else:
                if re.findall(r'%s' % keyw.lower(), annotation):
                    return True
        return False
    
    def read_keyword_file(self):
        '''reads user provided keywords for domain selection'''
        self._keyword_list = [rna_keyword.strip()
                              for rna_keyword in open(
                                self._keywords_file, 'r')]
        return self._keyword_list
    
    def read_pfam_domain_file(self):
        ''''''
        with open(self._pfam_domain_file, 'r') as in_fh:
            try:
                for entry in in_fh:
                    pfam_id = entry.split('\t')[0]
                    family_id = entry.split('\t')[1]
                    short_name = entry.split('\t')[3]
                    full_name = entry.split('\t')[8]
                    for check_keyword in self._keyword_list:
                        new_keyword = check_keyword.replace(
                            '-', ' ').replace('_', ' ')
                        if ' like' in new_keyword:
                            new_keyword = new_keyword.replace(
                                ' like', ' ').strip().replace('  ', ' ')
                        for search_annotation in list(
                            [family_id, short_name, full_name]):
                            if self._string_search(new_keyword, 
                            search_annotation):
                                self._pfam_domain_dict.setdefault(
                                    check_keyword, set()).add(pfam_id)
            except UnicodeDecodeError:
                pass
        return self._pfam_domain_dict
   
    def read_interpro_mapped_cdd_file(self):
        '''Parses cdd interpro mapped file for
        the extraction of common information'''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                ipr_id = entry.strip().split('\t')[0]
                ipr_members = entry.strip().split('\t')[1]
                cdd_id = entry.strip().split('\t')[2]
                cdd_member = entry.strip().split('\t')[3]
                domain_length = entry.strip().split('\t')[4]
                self._mapped_cdd_members[cdd_member] = ipr_members
                if '|' in ipr_members:
                    for each_ipr_member in ipr_members.split('|'):
                        self._mapped_ipr_legnth[each_ipr_member] = domain_length
        self._mapped_cdd_members, self._mapped_ipr_legnth
    
    def map_cdd_data_to_keywords(self):
        '''Parses CDD annotation data from table'''
        with open(self._cdd_whole_data_file, 'r') as cdd_data_fh:
            for each_entry in cdd_data_fh:
                cdd_id = each_entry.split('\t')[0]
                domain_id = each_entry.split('\t')[1]
                each_entry = each_entry.replace(each_entry.split('\t')[3], " ".join(
                        each_entry.split('\t')[3].split())).replace(';', ',')
                if 'smart' in domain_id:
                    each_entry = each_entry.replace(
                        domain_id, 'SM'+domain_id.split('smart')[-1])
                    domain_id = 'SM'+domain_id.split('smart')[-1]
                if 'pfam' in domain_id:
                    each_entry = each_entry.replace(
                        domain_id, 'PF'+domain_id.split('pfam')[-1])
                    domain_id = 'PF'+domain_id.split('pfam')[-1]
                if domain_id in self._mapped_cdd_members.keys():
                    cdd_entry = "%s\t%s\t%s" % (
                        '\t'.join(each_entry.split('\t')[0:-1]),
                            self._mapped_cdd_members[domain_id], 
                            each_entry.split('\t')[-1])
                else:
                    cdd_entry = "%s\tNA\t%s" % (
                        '\t'.join(each_entry.split('\t')[0:-1]),
                                                each_entry.split('\t')[-1])
                for check_keyword in self._keyword_list:
                    try:
                        if domain_id in self._pfam_domain_dict[check_keyword]:
                            self._selected_cdd_entry_dict.setdefault(
                                check_keyword, set()).add(cdd_entry.strip())
                    except KeyError:
                        pass
                    else:
                        new_keyword = check_keyword.replace(
                            '-', ' ').replace('_', ' ')
                        if ' like' in new_keyword:
                            new_keyword = new_keyword.replace(
                                ' like', ' ').replace('  ', ' ')
                        for search_annotation in cdd_entry.strip().split(
                            '\t')[0:4]:
                            if self._string_search(new_keyword, 
                            search_annotation):
                                self._selected_cdd_entry_dict.setdefault(
                                    check_keyword, set()).add(cdd_entry.strip())
        self.create_keyword_selected_domain_file('cdd', 
        self._selected_cdd_entry_dict, self._cdd_domain_path)
    
    def map_ipr_data_to_keywords(self):
        '''Parses InterPro annotation data from table'''
        with open(self._interpro_whole_data_file, 'r') as in_fh:
            for ipr_entry in in_fh:
                domain_id = ipr_entry.split('\t')[1]
                try:
                    if domain_id in self._mapped_ipr_legnth.keys():
                        mapped_length = self._mapped_ipr_legnth[domain_id]
                    else:
                        mapped_length = 'NA'
                except:
                    mapped_length = 'NA'
                for check_keyword in self._keyword_list:
                    try:
                        if domain_id in self._pfam_domain_dict[check_keyword]:
                            self._selected_ipr_entry_dict.setdefault(
                                check_keyword, set()).add(
                                '%s\t%s' % (ipr_entry.strip(), mapped_length))
                    except KeyError:
                        pass
                    else:
                        new_keyword = check_keyword.replace(
                            '-', ' ').replace('_', ' ')
                        if ' like' in new_keyword:
                            new_keyword = new_keyword.replace(
                                ' like', ' ').replace('  ', ' ')
                        for search_annotation in ipr_entry.strip().split('\t')[0:4]:
                            if self._string_search(new_keyword, search_annotation):
                                self._selected_ipr_entry_dict.setdefault(
                                    check_keyword, set()).add(
                                    '%s\t%s' % (ipr_entry.strip(), mapped_length))
        self.create_keyword_selected_domain_file('interpro', 
        self._selected_ipr_entry_dict, self._ipr_domain_path)
    
    def create_keyword_selected_domain_file(self, domain_db, 
    selected_domain_entry_dict, domain_data_path):
        '''Creates a file with CDD or interpro derived domain of interest'''
        keyword_selected_domain_dict = {}
        for keyword in self._keyword_list:
            with open(domain_data_path+'/'+keyword+'_related_'+domain_db+'_domains.tab',
                      'w') as selected_domain_file:
                if keyword in selected_domain_entry_dict.keys():
                    for each_entry in selected_domain_entry_dict[keyword]:
                        keyword_selected_domain_dict.setdefault(
                            each_entry, set()).add(keyword)
                        if '#'in keyword:
                            keyword = keyword.replace('#', '')
                        selected_domain_file.write("%s\t%s\n" % (
                            each_entry.strip(), keyword))
        with open(domain_data_path+'/all_keyword_selected_'+domain_db+'_data.tab',
                  'w') as keyword_selected_domain_file:
            for domain_entry in keyword_selected_domain_dict.keys():
                all_keywords = ', '.join(list(keyword_selected_domain_dict[
                    domain_entry]))
                keyword_selected_domain_file.write('%s\t%s\n' % (
                    domain_entry.strip(), all_keywords))



#!/usr/bin/env python
# Description = Calculates literature based secondary structures derived
# from residue information

import os
from collections import defaultdict


class LiteratureSecstrAnalysis(object):
    def __init__(self, selected_proteins,
                 fasta_path, outpath):
        self._selected_proteins = selected_proteins
        self._fasta_path = fasta_path
        self._outpath = outpath
        
        self._selected_protein_set = set()
        self._literature_acid_str = {
            'H': ['E', 'A', 'L', 'M', 'Q', 'K', 'R', 'H', 'Z'],
            'E': ['V', 'I', 'Y', 'C', 'W', 'F', 'T'],
            'C': ['G', 'N', 'P', 'S', 'D', 'B']
        }
        # 1: alpha-helix (H), 2: beta-sheet(E), 3: turns and loops(C)
        # https://en.wikibooks.org/wiki/Structural_Biochemistry/Proteins/Structures
        
    def streamline_literature_secstr_analysis(self):
        '''To call from apricot'''
        self.parse_selected_data()
        self.run_literature_ss_analysis()
        self.create_job_completion_file()
        
    def parse_selected_data(self):
        '''Parses selected data for uid'''
        with open(self._selected_proteins, 'r') as in_fh:
            for entry in in_fh:
                if not entry.startswith('Entry'):
                    self._selected_protein_set.add(entry.split('\t')[0])
        return self._selected_protein_set
    
    def run_literature_ss_analysis(self):
        '''Runs Literature based  3-state secondary
        structure predictionon the selected uids'''
        for files in os.listdir(self._fasta_path):
            if files.split('.')[0] in self._selected_protein_set:
                print("Literature acid based secondary structure "
                      "analysis for %s" % files)
                with open(self._outpath+'/%s.plain' % files.split('.')[0],
                          'w') as out_fh:
                    out_fh.write("#Literature based three-class secondary "
                                 "structure prediction results\n\n")
                    with open("%s/%s" % (
                            self._fasta_path, files), 'r') as in_fh:
                        literature_acid_seq = []
                        for entry in in_fh:
                            if not entry.startswith('>'):
                                literature_acid_seq.append(entry.strip())
                        str_list = []
                        seq_dict = defaultdict(
                            lambda: defaultdict(lambda: defaultdict()))
                        for aa in ''.join(literature_acid_seq):
                            str_val = ''
                            if aa == 'J':
                                str_val = 'x'
                            elif aa == 'X':
                                str_val = 'x'
                            else:
                                for sec_str in self._literature_acid_str.keys():
                                    if aa in self._literature_acid_str[sec_str]:
                                        str_val = sec_str
                            str_list.append(str_val)
                        full_seq = self._split_str(
                            ''.join(literature_acid_seq), 60)
                        full_pred = self._split_str(''.join(str_list), 60)
                        for i, seq in enumerate(full_pred):
                            aa_seq = '  AA: %s' % (full_seq[i])
                            pred = 'Pred: %s' % (full_pred[i])
                            out_fh.write("%s\n%s\n\n" % (pred, aa_seq))
                            
    def _split_str(self, seq, chunk, skip_tail=False):
        lst = []
        if chunk <= len(seq):
            lst.extend([seq[:chunk]])
            lst.extend(self._split_str(seq[chunk:], chunk, skip_tail))
        elif not skip_tail and seq:
            lst.extend([seq])
        return lst
    
    def create_job_completion_file(self):
        with open(self._outpath+'/literature_ss_analysis.txt', 'w') as out_fh:
            out_fh.write("Literature based secondary structures for the "
                         "selected proteins are generated.\n")
            out_fh.write("The files generated by the analysis:.\n")
            out_fh.write('\n'.join(os.listdir(self._outpath)))



#!/usr/bin/env python
# Description = selects rna related domains from InterPro database.

import re


class RnaRelatedIPRSelection(object):
    def __init__(self, keywords_file,
                 ipr_whole_data_file,
                 interpro_mapped_cdd,
                 domain_data_path):
        self._keywords_file = keywords_file
        self._ipr_whole_data_file = ipr_whole_data_file
        self._interpro_mapped_cdd = interpro_mapped_cdd
        self._domain_data_path = domain_data_path
        
        self.ipr_whole_data_list = []
        self._ipr_dict = {}
        self._mapped_interpro_length = {}
    
    def select_ipr_domains(self):
        '''To call from apricot'''
        self.read_keyword_file()
        self.read_interpro_mapped_cdd_file()
        self.read_ipr_whole_data_file()
        self.create_keyword_selected_domain_file()
        
    def read_keyword_file(self):
        '''reads user provided keywords for domain selection'''
        self._keyword_list = [
            rna_keyword.strip() for rna_keyword in open(
                self._keywords_file)]
        return self._keyword_list
    
    def read_interpro_mapped_cdd_file(self):
        '''Parses cdd interpro mapped file for
        the extraction of common information'''
        with open(self._interpro_mapped_cdd, 'r') as in_fh:
            for entry in in_fh:
                cdd_member = entry.split('\t')[3]
                domain_length = entry.split('\t')[4].strip()
                self._mapped_interpro_length[cdd_member] = domain_length
        self._mapped_interpro_length
        
    def read_ipr_whole_data_file(self):
        '''Parses InterPro annotation data from table'''
        with open(self._ipr_whole_data_file, 'r') as in_fh:
            for ipr_id_entry in in_fh:
                self.ipr_whole_data_list.append(ipr_id_entry.strip())
                self._ipr_dict[ipr_id_entry.strip()] = ipr_id_entry.strip(
                    ).split('\t')[2]
        return self.ipr_whole_data_list, self._ipr_dict
        
    def create_keyword_selected_domain_file(self):
        '''Creates a file with InterPro derived domain of interest'''
        self._keyword_annotation_dict = {}
        self._keyword_selected_domain = {}
        for ipr_entry in self.ipr_whole_data_list:
            domain_full_name = list(self._ipr_dict[
                ipr_entry].lower().replace('-', ' ').replace(
                    '_', ' ').split(' '))
            for keyword in self._keyword_list:
                check_keyword = keyword.lower()
                if '-' in check_keyword:
                    ref_kw_list = check_keyword.split('-')
                    new_kw_list = []
                    for domain_name in domain_full_name:
                        for string in ref_kw_list:
                            if string == 'rna':
                                if re.findall(
                                        r"(?:\s|^)%s(?=\s|$)" % '*rna',
                                        domain_name):
                                    if not string in set(new_kw_list):
                                        new_kw_list.append(string)
                            else:
                                if re.findall(
                                        r"(?:\s|^)%s(?=\s|$)" % string,
                                        domain_name):
                                    if not string in set(new_kw_list):
                                        new_kw_list.append(string)
                    if str(ref_kw_list) == str(new_kw_list):
                        self._keyword_annotation_dict.setdefault(
                            keyword, []).append(ipr_entry)
                else:
                    for domain_name in domain_full_name:
                        if re.findall(
                                r"(?:\s|^)%s(?=\s|$)" % check_keyword,
                                domain_name):
                            self._keyword_annotation_dict.setdefault(
                                keyword, []).append(ipr_entry)
        for fkeyword in self._keyword_list:
            fkeyword = fkeyword.replace(' ', '_')
            with open(self._domain_data_path+'/'+fkeyword+'_related_ipr_ids.tab',
                      'w') as key_fh:
                if self._keyword_annotation_dict.get(fkeyword):
                    for each_entry in self._keyword_annotation_dict[fkeyword]:
                        each_entry = each_entry.replace(each_entry.split(
                            '\t')[3], " ".join(each_entry.split('\t')[3].split(
                            ))).replace(';', ',')
                        cdd_id = each_entry.split('\t')[1]
                        if cdd_id in set(self._mapped_interpro_length.keys()):
                            length = self._mapped_interpro_length[cdd_id]
                        else:
                            length = 'NA'
                        self._keyword_selected_domain.setdefault("%s\t%s" % (
                            each_entry, length), set()).add(fkeyword)
                        key_fh.write("%s\t%s\t%s\n" % (
                            each_entry, length, fkeyword))
        with open(self._domain_data_path+'/all_keyword_selected_ipr_data.tab',
                  'w') as keyword_selected_domain_file:
            for domain_entry in self._keyword_selected_domain.keys():
                keywords = ', '.join(list(
                    self._keyword_selected_domain[domain_entry]))
                keyword_selected_domain_file.write('%s\t%s\n' % (
                    domain_entry, keywords))



#!/usr/bin/env python
# Description = Download the taxonomy ids related to user provided species


def select_taxids(species, reference_taxonomy_file, selected_taxonomy_file):
    '''Selects taxonomy ids for the query species'''
    if not str(species) == 'None':
        parse_uniprot_tax_file(
            species, reference_taxonomy_file, selected_taxonomy_file)
    else:
        parse_uniprot_tax_file(
            'N=', reference_taxonomy_file, selected_taxonomy_file)

        
def parse_uniprot_tax_file(
        species, reference_taxonomy_file, selected_taxonomy_file):
    '''Parses the taxonomy id file from UniProt for all the species'''
    with open(selected_taxonomy_file, 'w') as out_fh:
        out_fh.write("Official (scientific) name\tTaxonomy ID\n")
        with open(reference_taxonomy_file, 'r') as in_fh:
            for entry in in_fh:
                if 'N=' in entry:
                    if ',' in species:
                        for each_species in species.split(','):
                            if each_species.lower() in entry.lower():
                                tax_id = entry.split(':')[0].split(' ')[-1]
                                species_name = entry.strip().split('=')[1]
                                out_fh.write("%s\t%s\n" % (
                                    species_name, tax_id))
                    else:
                        if species.lower() in entry.lower():
                            tax_id = entry.split(':')[0].split(' ')[-1]
                            species_name = entry.strip().split('=')[1]
                            out_fh.write("%s\t%s\n" % (species_name, tax_id))
